import os
import pygame
from pygame import Rect
from pygame.math import Vector2

os.environ['SDL_VIDEO_CENTERED'] = '1'

class Unit():
    def __init__(self,state,position,tile):
        self.state = state
        self.position = position
        self.tile = tile
    def move(self,moveVector):
        raise NotImplementedError()

class Tank(Unit):
    def move(self,moveVector):
        # Compute new tank position
        newTankPos = self.position + moveVector

        # Don't allow positions outside the world
        if newTankPos.x < 0 or newTankPos.x >= self.state.worldSize.x \
        or newTankPos.y < 0 or newTankPos.y >= self.state.worldSize.y:
            return

        # Don't allow tower positions 
        for unit in self.state.units:
            if newTankPos == unit.position:
                return

        self.position = newTankPos

class Tower(Unit):
    def move(self,moveVector):
        pass

class GameState():
    def __init__(self):
        self.worldSize = Vector2(16,10)
        self.units = [
            Tank(self,Vector2(5,4),Vector2(1,0)),
            Tower(self,Vector2(10,3),Vector2(0,1)),
            Tower(self,Vector2(10,5),Vector2(0,1))
        ]

    def update(self,moveTankCommand):
        for unit in self.units:
            unit.move(moveTankCommand)

class UserInterface():
    def __init__(self):
        pygame.init()

        # Game state
        self.gameState = GameState()

        # Rendering properties
        self.cellSize = Vector2(64,64)
        self.unitsTexture = pygame.image.load("units.png")

        # Window
        windowSize = self.gameState.worldSize.elementwise() * self.cellSize
        self.window = pygame.display.set_mode((int(windowSize.x),int(windowSize.y)))
        pygame.display.set_caption("Discover Python & Patterns - https://www.patternsgameprog.com")
        pygame.display.set_icon(pygame.image.load("icon.png"))
        self.moveTankCommand = Vector2(0,0)

        # Loop properties
        self.clock = pygame.time.Clock()
        self.running = True

    def processInput(self):
        self.moveTankCommand = Vector2(0,0)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.running = False
                break
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.running = False
                    break
                elif event.key == pygame.K_RIGHT:
                    self.moveTankCommand.x = 1
                elif event.key == pygame.K_LEFT:
                    self.moveTankCommand.x = -1
                elif event.key == pygame.K_DOWN:
                    self.moveTankCommand.y = 1
                elif event.key == pygame.K_UP:
                    self.moveTankCommand.y = -1

    def update(self):
        self.gameState.update(self.moveTankCommand)

    def renderUnit(self,unit):
        # Location on screen
        spritePoint = unit.position.elementwise()*self.cellSize

        # Unit texture
        texturePoint = unit.tile.elementwise()*self.cellSize
        textureRect = Rect(int(texturePoint.x), int(texturePoint.y), int(self.cellSize.x),int(self.cellSize.y))
        self.window.blit(self.unitsTexture,spritePoint,textureRect)

        # Weapon texure
        texturePoint = Vector2(0,6).elementwise()*self.cellSize
        textureRect = Rect(int(texturePoint.x), int(texturePoint.y), int(self.cellSize.x),int(self.cellSize.y))
        self.window.blit(self.unitsTexture,spritePoint,textureRect)    

    def render(self):
        self.window.fill((0,0,0))

        for unit in self.gameState.units:
            self.renderUnit(unit)

        pygame.display.update()    

    def run(self):
        while self.running:
            self.processInput()
            self.update()
            self.render()
            self.clock.tick(60)

userInterface = UserInterface()
userInterface.run()

pygame.quit()